# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)


DESCRIPTOR = descriptor.FileDescriptor(
  name='kegnet.proto',
  package='',
  serialized_pb='\n\x0ckegnet.proto\"\x06\n\x04Ping\"\x0b\n\tQuitEvent\"\x14\n\x12StartCompleteEvent\"0\n\x0bMeterUpdate\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x0f\n\x07reading\x18\x02 \x02(\x02\"\x88\x02\n\nFlowUpdate\x12\x0f\n\x07\x66low_id\x18\x01 \x02(\r\x12\x10\n\x08tap_name\x18\x02 \x02(\t\x12$\n\x05state\x18\x03 \x02(\x0e\x32\x15.FlowUpdate.FlowState\x12\x10\n\x08username\x18\x04 \x01(\t\x12\x12\n\nstart_time\x18\x05 \x02(\r\x12\x1a\n\x12last_activity_time\x18\x06 \x02(\r\x12\r\n\x05ticks\x18\x07 \x02(\r\x12\x11\n\tvolume_ml\x18\x08 \x02(\x02\"M\n\tFlowState\x12\x0b\n\x07INITIAL\x10\x01\x12\n\n\x06\x41\x43TIVE\x10\x02\x12\x08\n\x04IDLE\x10\x03\x12\x0e\n\nCLOSE_WAIT\x10\x04\x12\r\n\tCOMPLETED\x10\x05\" \n\x0cTapIdleEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\"\x80\x01\n\x11\x44rinkCreatedEvent\x12\x0f\n\x07\x66low_id\x18\x01 \x02(\r\x12\x10\n\x08\x64rink_id\x18\x02 \x02(\r\x12\x10\n\x08tap_name\x18\x03 \x02(\t\x12\x12\n\nstart_time\x18\x04 \x02(\r\x12\x10\n\x08\x65nd_time\x18\x05 \x02(\r\x12\x10\n\x08username\x18\x06 \x01(\t\"\xa3\x01\n\x0eTokenAuthEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x18\n\x10\x61uth_device_name\x18\x02 \x02(\t\x12\x13\n\x0btoken_value\x18\x03 \x02(\t\x12*\n\x06status\x18\x04 \x02(\x0e\x32\x1a.TokenAuthEvent.TokenState\"$\n\nTokenState\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\x0b\n\x07REMOVED\x10\x02\"\x82\x01\n\rUserAuthEvent\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12\x11\n\tuser_name\x18\x02 \x02(\t\x12\'\n\x05state\x18\x03 \x02(\x0e\x32\x18.UserAuthEvent.UserState\"#\n\tUserState\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\x0b\n\x07REMOVED\x10\x02\"8\n\x0bThermoEvent\x12\x13\n\x0bsensor_name\x18\x01 \x02(\t\x12\x14\n\x0csensor_value\x18\x02 \x02(\x02\"\x81\x01\n\x0b\x46lowRequest\x12\x10\n\x08tap_name\x18\x01 \x02(\t\x12$\n\x07request\x18\x02 \x02(\x0e\x32\x13.FlowRequest.Action\":\n\x06\x41\x63tion\x12\x0e\n\nSTART_FLOW\x10\x01\x12\r\n\tSTOP_FLOW\x10\x02\x12\x11\n\rREPORT_STATUS\x10\x03\"\x16\n\x14HeartbeatSecondEvent\"\x16\n\x14HeartbeatMinuteEvent\"\x14\n\x12HeartbeatHourEvent\"4\n\x10\x43reditAddedEvent\x12\x0e\n\x06\x61mount\x18\x01 \x02(\x02\x12\x10\n\x08username\x18\x02 \x01(\t')



_FLOWUPDATE_FLOWSTATE = descriptor.EnumDescriptor(
  name='FlowState',
  full_name='FlowUpdate.FlowState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='INITIAL', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='ACTIVE', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='IDLE', index=2, number=3,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='CLOSE_WAIT', index=3, number=4,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='COMPLETED', index=4, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=297,
  serialized_end=374,
)

_TOKENAUTHEVENT_TOKENSTATE = descriptor.EnumDescriptor(
  name='TokenState',
  full_name='TokenAuthEvent.TokenState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='ADDED', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REMOVED', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=669,
  serialized_end=705,
)

_USERAUTHEVENT_USERSTATE = descriptor.EnumDescriptor(
  name='UserState',
  full_name='UserAuthEvent.UserState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='ADDED', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REMOVED', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=803,
  serialized_end=838,
)

_FLOWREQUEST_ACTION = descriptor.EnumDescriptor(
  name='Action',
  full_name='FlowRequest.Action',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='START_FLOW', index=0, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='STOP_FLOW', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='REPORT_STATUS', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=970,
  serialized_end=1028,
)


_PING = descriptor.Descriptor(
  name='Ping',
  full_name='Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=16,
  serialized_end=22,
)


_QUITEVENT = descriptor.Descriptor(
  name='QuitEvent',
  full_name='QuitEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=24,
  serialized_end=35,
)


_STARTCOMPLETEEVENT = descriptor.Descriptor(
  name='StartCompleteEvent',
  full_name='StartCompleteEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=37,
  serialized_end=57,
)


_METERUPDATE = descriptor.Descriptor(
  name='MeterUpdate',
  full_name='MeterUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='MeterUpdate.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='reading', full_name='MeterUpdate.reading', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=59,
  serialized_end=107,
)


_FLOWUPDATE = descriptor.Descriptor(
  name='FlowUpdate',
  full_name='FlowUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='flow_id', full_name='FlowUpdate.flow_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='tap_name', full_name='FlowUpdate.tap_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='state', full_name='FlowUpdate.state', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='FlowUpdate.username', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='start_time', full_name='FlowUpdate.start_time', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='last_activity_time', full_name='FlowUpdate.last_activity_time', index=5,
      number=6, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='ticks', full_name='FlowUpdate.ticks', index=6,
      number=7, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='volume_ml', full_name='FlowUpdate.volume_ml', index=7,
      number=8, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FLOWUPDATE_FLOWSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=110,
  serialized_end=374,
)


_TAPIDLEEVENT = descriptor.Descriptor(
  name='TapIdleEvent',
  full_name='TapIdleEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='TapIdleEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=376,
  serialized_end=408,
)


_DRINKCREATEDEVENT = descriptor.Descriptor(
  name='DrinkCreatedEvent',
  full_name='DrinkCreatedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='flow_id', full_name='DrinkCreatedEvent.flow_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='drink_id', full_name='DrinkCreatedEvent.drink_id', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='tap_name', full_name='DrinkCreatedEvent.tap_name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='start_time', full_name='DrinkCreatedEvent.start_time', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='end_time', full_name='DrinkCreatedEvent.end_time', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='DrinkCreatedEvent.username', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=411,
  serialized_end=539,
)


_TOKENAUTHEVENT = descriptor.Descriptor(
  name='TokenAuthEvent',
  full_name='TokenAuthEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='TokenAuthEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='auth_device_name', full_name='TokenAuthEvent.auth_device_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='token_value', full_name='TokenAuthEvent.token_value', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='status', full_name='TokenAuthEvent.status', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TOKENAUTHEVENT_TOKENSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=542,
  serialized_end=705,
)


_USERAUTHEVENT = descriptor.Descriptor(
  name='UserAuthEvent',
  full_name='UserAuthEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='UserAuthEvent.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='user_name', full_name='UserAuthEvent.user_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='state', full_name='UserAuthEvent.state', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _USERAUTHEVENT_USERSTATE,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=708,
  serialized_end=838,
)


_THERMOEVENT = descriptor.Descriptor(
  name='ThermoEvent',
  full_name='ThermoEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='sensor_name', full_name='ThermoEvent.sensor_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sensor_value', full_name='ThermoEvent.sensor_value', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=840,
  serialized_end=896,
)


_FLOWREQUEST = descriptor.Descriptor(
  name='FlowRequest',
  full_name='FlowRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='tap_name', full_name='FlowRequest.tap_name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='request', full_name='FlowRequest.request', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FLOWREQUEST_ACTION,
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=899,
  serialized_end=1028,
)


_HEARTBEATSECONDEVENT = descriptor.Descriptor(
  name='HeartbeatSecondEvent',
  full_name='HeartbeatSecondEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1030,
  serialized_end=1052,
)


_HEARTBEATMINUTEEVENT = descriptor.Descriptor(
  name='HeartbeatMinuteEvent',
  full_name='HeartbeatMinuteEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1054,
  serialized_end=1076,
)


_HEARTBEATHOUREVENT = descriptor.Descriptor(
  name='HeartbeatHourEvent',
  full_name='HeartbeatHourEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1078,
  serialized_end=1098,
)


_CREDITADDEDEVENT = descriptor.Descriptor(
  name='CreditAddedEvent',
  full_name='CreditAddedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='amount', full_name='CreditAddedEvent.amount', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='username', full_name='CreditAddedEvent.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=1100,
  serialized_end=1152,
)


_FLOWUPDATE.fields_by_name['state'].enum_type = _FLOWUPDATE_FLOWSTATE
_FLOWUPDATE_FLOWSTATE.containing_type = _FLOWUPDATE;
_TOKENAUTHEVENT.fields_by_name['status'].enum_type = _TOKENAUTHEVENT_TOKENSTATE
_TOKENAUTHEVENT_TOKENSTATE.containing_type = _TOKENAUTHEVENT;
_USERAUTHEVENT.fields_by_name['state'].enum_type = _USERAUTHEVENT_USERSTATE
_USERAUTHEVENT_USERSTATE.containing_type = _USERAUTHEVENT;
_FLOWREQUEST.fields_by_name['request'].enum_type = _FLOWREQUEST_ACTION
_FLOWREQUEST_ACTION.containing_type = _FLOWREQUEST;

class Ping(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _PING
  
  # @@protoc_insertion_point(class_scope:Ping)

class QuitEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _QUITEVENT
  
  # @@protoc_insertion_point(class_scope:QuitEvent)

class StartCompleteEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _STARTCOMPLETEEVENT
  
  # @@protoc_insertion_point(class_scope:StartCompleteEvent)

class MeterUpdate(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _METERUPDATE
  
  # @@protoc_insertion_point(class_scope:MeterUpdate)

class FlowUpdate(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FLOWUPDATE
  
  # @@protoc_insertion_point(class_scope:FlowUpdate)

class TapIdleEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TAPIDLEEVENT
  
  # @@protoc_insertion_point(class_scope:TapIdleEvent)

class DrinkCreatedEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DRINKCREATEDEVENT
  
  # @@protoc_insertion_point(class_scope:DrinkCreatedEvent)

class TokenAuthEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TOKENAUTHEVENT
  
  # @@protoc_insertion_point(class_scope:TokenAuthEvent)

class UserAuthEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _USERAUTHEVENT
  
  # @@protoc_insertion_point(class_scope:UserAuthEvent)

class ThermoEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _THERMOEVENT
  
  # @@protoc_insertion_point(class_scope:ThermoEvent)

class FlowRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _FLOWREQUEST
  
  # @@protoc_insertion_point(class_scope:FlowRequest)

class HeartbeatSecondEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATSECONDEVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatSecondEvent)

class HeartbeatMinuteEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATMINUTEEVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatMinuteEvent)

class HeartbeatHourEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATHOUREVENT
  
  # @@protoc_insertion_point(class_scope:HeartbeatHourEvent)

class CreditAddedEvent(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CREDITADDEDEVENT
  
  # @@protoc_insertion_point(class_scope:CreditAddedEvent)

# @@protoc_insertion_point(module_scope)
