{% extends "kegadmin/base.html" %}
{% load kegweblib %}

{% block title %}tap admin{% endblock %}

{% block kegadmin-main %}
<div class="kb-contentbox">
<h1>edit taps</h1>
coming soon.
</div>
{% endblock %}
