{% extends "kegadmin/base.html" %}
{% load kegweblib %}

{% block title %}tap admin{% endblock %}

{% block kegadmin-main %}
<div class="kb-contentbox">
  <h1>kegadmin: main</h1>
  <p>
    this page intentionally left blank.
  </p>
</div>
{% endblock %}
