/* @group Main Body */

body {
	background-color: #fff;
	font-family: "Lucida Grande", Lucida, Verdana, sans-serif;
	color: black;
  margin: 0px;
  background-image: url(http://kegbot.org/kmedia/images/kegs-corner.png);
  background-repeat: no-repeat;
}

/* @end */

/* @group Links */

a:link {
	color: navy;
	font-weight: bold;
	text-decoration: none;
}

a:visited {
	color: navy;
	font-weight: bold;
	text-decoration: none;
}

a:active {
	color: navy;
	font-weight: bold;
	text-decoration: none;
}

a:hover {
	color: navy;
	font-weight: bold;
	text-decoration: none;
}



/* @end */

/* @group Kegweb Elements */

#kb-main-page {
  margin-left: auto;
  margin-right: auto;
  width: 900px;
}

#kb-main-container {
	width: 900px;
}

#kb-main-header {
	border-bottom: 1px solid gray;
	background-color: #00B1FE;
	margin-left: 0px;
	margin-top: 0px;
	margin-bottom: 16px;
  width: 900px;
  border-bottom: 2px solid #007BFE;
}

#kb-main-header-text {
  font: bold 280%/100% "Lucida Grande", Arial, sans-serif;
	color: #1A5568;
  margin: 16px;
}

#kb-main-header-links {
	font: bold 12px "Lucida Grande", Lucida, Verdana, sans-serif;
	color: gray;
  margin-left: 16px;
}

.kb-header-link {
  height: 16px;
  float: left;
  clear: all;
  border-left: 1px solid #000;
  padding-left: 2px;
  padding-right: 16px;
}

.kb-main-content {
	background-color: white;
  padding-top: 16px;
  border: 1px solid #ACB6C0;
	margin-bottom: 16px;
}

#kb-footer {
	font: 8px "Lucida Grande", Lucida, Verdana, sans-serif;
  color: #C3C0C0;
  text-align: center;
  margin-top: 8px;
}

/* @end */

.kb-tapbox {
  width: 480px;
  border: 1px solid #C3C0C0;
  margin-top: 8px;
  margin-bottom: 8px;
}

.kb-tapbox-header {
  background: #eee;
  padding: 4px;
}

.kb-tapbox-content {
  padding: 4px;
}

.kb-tapbox-image {
  min-width: 96px;
  min-height: 96px;
  border: 1px dotted #C3C0C0;
  margin-top: auto;
  float: left;
}

.kb-tapbox-detail {
  font: 80%/100% "Lucida Grande", Arial, sans-serif;
  margin-left: 12px;
  float: left;
  width: 360px;
}

.kb-tapbox-footer {
	border-top: 1px solid #C3C0C0;
	font: 12px "Lucida Grande", Lucida, Verdana, sans-serif;
	text-align: right;
	width: 100%;
	margin-top: 4px;
	padding: 1px;
}

.kb-main-content h1 {
  font: bold 200%/100% "Lucida Grande", Arial, sans-serif;
  position: relative;
  margin: 0;
  color: #000;
  letter-spacing: -2px
  margin-bottom: 16px;
}

.kb-main-content h2 {
  font: bold 120%/80% "Lucida Grande", Arial, sans-serif;
  position: relative;
  margin: 0;
  color: #000;
  letter-spacing: -2px
  margin-bottom: 16px;
}

.kb-twocol-left {
  width: 500px;
  height: 100%;
  float: left;
  margin-right: 8px;
}

.kb-twocol-right {
  border-left: 1px #C3C0C0 dotted;
  height: 100%;
  float: left;
}


.kb-contentbox-head {
  font: bold 200%/100% "Lucida Grande", Arial, sans-serif;
  position: relative;
  margin: 0;
  color: #000;
  letter-spacing: -2px
  margin-bottom: 16px;
	padding-left: 16px; /* match kb-main-header */
}

.kb-contentbox {
	padding-left: 16px; /* match kb-main-header */
}

.kb-statstable {
  font: 80%/100% "Lucida Grande", Arial, sans-serif;
}

.kb-statstable th {
  text-align: right;
  margin-right: 4px;
}

.kb-drink-box {
  width: 300px;
  margin-bottom: 8px;
  padding: 8px;
  border-bottom: 1px dotted #007BFE;
  height: 50px;
}

.kb-drink-box-mugshot {
  float: left;
  margin-right: 8px;
}

.kb-drink-box-details {
  margin-left: 8px;
}

.kb-drink-box-details-headline {
}

.kb-drink-box-details-keginfo {
  font-size: 0.7em;
  color: #999;
}

.kb-drink-box-details-keginfo a:link,
.kb-drink-box-details-keginfo a:visited {
  color: #999;
  text-decoration: underline;
  font-weight: normal;
}

.kb-drink-box-details-dateline {
  font-size: 0.7em;
  color: #ccc;
  margin-top: 4px;
  font-style: italic;
  text-align: right;
}

.kb-chart-missing {
  font-size: 0.6em;
  color: #aaf;
  border: 1px dotted #aaf;
  margin: 0px;
  text-align: center;
}

.clear {
  clear:both;
  height:1px;
  overflow:hidden;
}

/* form stuff */
.kb-form {
  font-size: 0.8em;
}

.kb-form th {
  text-align: right;
  vertical-align: top;
}

.kb-form tr {
}

.kb-form td {
  color: #888;
  font-size: 0.7em;
}

ul.errorlist {
  color: #bb3333;
  font-size: 11px;
  font-weight: bold;
  list-style-type: none;
  margin: 0px;
}
ul.errorlist li {
  margin-left: 0px;
}

div.plainboxhead {
   border: 1px solid #333;
   border-bottom: 0px;
   padding: 5px;
   background: #369;
}
div.plainbox {
   border: 1px solid #333;
   padding: 5px;
   background: #ccd;
}

.kb-keg-session {
  width: 460px;
  margin-bottom: 8px;
  padding: 8px;
  background: #b2def2;
  border: 1px solid #277394;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
}

.kb-session-box-headline {
  font-size: 1.2em;
  font-weight: bold;
}

.kb-session-box-detail {
  padding-left: 16px;
  padding-right: 16px;
  font: normal normal normal 80%/100% 'Lucida Grande', Arial, sans-serif;
}

.kb-session-box-summary {
  margin-bottom: 4px;
  background-color: #113341;
  color: #fff;
  font-size: 0.7em;
  padding: 3px;
}
.kb-session-box-summary a:link,
.kb-session-box-summary a:visited {
  color: #fff;
}

.kb-session-box-summary a:hover,
.kb-session-box-summary a:active {
  color: #ccf;
}

.kb-award-box {
  width: 48px;
  height: 48px;
  background: #4e4e4e;
  color: #eee;
  float: left;
  margin-right: 8px;
}

.kb-comment-box {
  font-size: 0.7em;
  width: 400px;
  margin-bottom: 4px;
  padding: 2px;
}

#kb-admin-nav {
  float: left;
  width: 160px;
  margin-right:32px;
  margin-bottom: 16px;
  font-size: 0.9em;
}

.kb-admin-nav-tab {
  width: 160px;
  height: 32px;
  line-height: 32px;
  background-color: #eee;
  margin-bottom: 2px;
  padding-left: 4px;
}

.kb-admin-nav-tab-hover {
  background-color: #aaa;
  cursor: pointer;
}

#kb-recent-pours {
  width: 340px;
  height: 300px;
  overflow: auto;
}

#kb-message-container {
  width: 900px;
  text-align: center;
  margin-bottom: 16px;
}

.kb-message-box {
  width: 100%;
  font-weight: bold;
  font-size: 0.7em;
  margin-left: auto;
  margin-right: auto;
  padding: 2px;
}

.kb-message-success {
  background-color: #99ED84;
  width: 100%;
  padding: 2px;
}

.kb-message-error {
  background-color: #FCA7A7;
  width: 100%;
  padding: 2px;
}


/* vim sw=2 ts=2 noet */
