{% extends "page-twocol.html" %}
{% load kegweblib %}

{% block title %}password reset{% endblock %}

{% block col-1 %}
<div class="kb-contentbox-head">
  forgot username/password?
</div>
<div class="kb-contentbox">
  <p>
    that's ok, we can get that reset for you &amp; drinking again in no time!
  </p>
  <form action="" method="POST">{% csrf_token %}
    <table class="kb-form">
      {{ form.as_table }}
      <tr>
        <td colspan="2">
          <input type="submit" value="mail me!" />
        </td>
      </tr>
    </table>
  </form>
</div>
{% endblock %}
