{% extends "base.html" %}
{% load kegweblib %}

{% block title %}registered!{% endblock %}

{% block content %}
<div class="kb-contentbox-head">
   done!
</div>
<div class="kb-contentbox">
  <p>
    begin drinking!
  </p>
  <p>
    ... or perhaps <a href="{% url kb-account-main %}">edit your profile</a>, or
    <a href="{% url account-mugshot %}">upload your mugshot</a>?
  </p>
</div>

{% endblock %}
