{% extends "account/base.html" %}

{% block kb-account-main %}
<div class="kb-contentbox-head">
  hello, {{user.username}}
</div>
<div class="kb-contentbox">
  <form action="" method="POST">{% csrf_token %}
    <table class="kb-form">
      {{ profile_form.as_table }}
      <tr>
        <td colspan="2">
          <input type="submit" value="update profile" />
        </td>
      </tr>
    </table>
  </form>
</div>
{% endblock %}
