{% extends "page-twocol.html" %}
{% load kegweblib %}

{% block title %}keg {{ keg.seqn }}{% endblock %}

{% block kb-extrajs %}
   {% if USE_FACEBOOK %}
   {% load facebook_tags %}
   {% facebook_js %}
   {% endif %}
{% endblock %}

{% block col-1 %}
<div class="kb-contentbox-head">
   account
</div>
<div class="kb-contentbox">

  <form action="" method="POST">{% csrf_token %}

    <table class="kb-form">
    {{ form.as_table }}
    <tr>
      <td colspan="2">
        <input type="submit" value="login" />
      </td>
    </tr>
    </table>

  </form>

  <p>
    <span style="font-size:0.8em;">
      (did you <a href="/accounts/password/reset/">forget your username or
      password?</a>)
    </span>
  </p>
</div>

{% endblock col-1 %}

{% block col-2 %}

  {% if USE_FACEBOOK %}
  <div class="kb-contentbox-head">
    login using facebook
  </div>
  <div class="kb-contentbox">
    {% facebook_button %}
  </div>
  <p/>
  {% endif %}

  {% if USE_TWITTER %}
  <div class="kb-contentbox-head">
    login using twitter
  </div>
  <div class="kb-contentbox">
    {% load twitter_tags %}
    {% twitter_button %}
  </div>
  {% endif %}

{% endblock %}
