{% load kegweblib %}
<div class="kb-drink-box">
  <div class="kb-drink-box-mugshot">
    {% mugshot_box drink.user 48 %}
  </div>
  <div class="kb-drink-box-details">
    <div class="kb-drink-box-details-headline">
      {% if drink.user %}
      <a href="{% url drinker drink.user.username %}">{{drink.user.username}}</a>
      {% else %}
      guest
      {% endif %}
      poured
      <b>{{ drink.Volume.ConvertTo.Ounce|floatformat:2 }} ounces</b>
    </div>
    <div class="kb-drink-box-details-keginfo">
      <a href="{% url drink drink.seqn %}">drink {{drink.seqn}}</a>{% if drink.keg %}
        from
        <a href="{% url keg drink.keg.seqn %}">{{drink.keg.type.name}}</a>
      {% endif %}
    </div>
  </div>
  <div class="kb-drink-box-details-dateline">
    {% timeago drink.endtime %}
  </div>
  <div class="clear"></div>
</div>
