{% extends "base.html" %}
{% load kegweblib %}

{% block title %}main page{% endblock %}

{% block content %}
<div class="kb-contentbox-head">
   register
</div>
<div class="kb-contentbox">
  <form action="/accounts/register/" method="POST"> {% csrf_token %}
    <table class="kb-form">
      {{ form.as_table }}
      <tr>
        <td colspan="2">
          <input type="submit" value="register!" />
        </td>
      </tr>
    </table>
  </form>
</div>

{% endblock %}
